==> v0.2.0.1480.zip: lidarr/src/NzbDrone.Test.Common/NzbDroneRunner.cs <==
﻿using System;
using System.Diagnostics;
using System.IO;
using System.Threading;
using System.Xml;
using System.Xml.Linq;
using System.Xml.XPath;
using NLog;
using NUnit.Framework;
using NzbDrone.Common.EnvironmentInfo;
using NzbDrone.Common.Processes;
using RestSharp;

namespace NzbDrone.Test.Common
{
    public class NzbDroneRunner
    {
        private readonly IProcessProvider _processProvider;
        private readonly IRestClient _restClient;
        private Process _nzbDroneProcess;

        public string AppData { get; private set; }
        public string ApiKey { get; private set; }

        public NzbDroneRunner(Logger logger, int port = 7878)
        {
            _processProvider = new ProcessProvider(logger);
            _restClient = new RestClient("http://localhost:7878/api");
        }

        public void Start()
        {
            AppData = Path.Combine(TestContext.CurrentContext.TestDirectory, "_intg_" + DateTime.Now.Ticks);

            var nzbdroneConsoleExe = "Radarr.Console.exe";

            if (OsInfo.IsNotWindows)
            {
                nzbdroneConsoleExe = "Radarr.exe";
            }

            if (BuildInfo.IsDebug)
            {
                Start(Path.Combine(TestContext.CurrentContext.TestDirectory, "..\\..\\..\\..\\..\\_output\\Radarr.Console.exe"));
            }
            else
            {
                Start(Path.Combine(TestContext.CurrentContext.TestDirectory, "bin", nzbdroneConsoleExe));
            }

            while (true)
            {
                _nzbDroneProcess.Refresh();

                if (_nzbDroneProcess.HasExited)
                {
                    Assert.Fail("Process has exited");
                }

                SetApiKey();

                var request = new RestRequest("system/status");
                request.AddHeader("Authorization", ApiKey);
                request.AddHeader("X-Api-Key", ApiKey);

                var statusCall = _restClient.Get(request);

                if (statusCall.ResponseStatus == ResponseStatus.Completed)
                {
                    Console.WriteLine("NzbDrone is started. Running Tests");
                    return;
                }

                Console.WriteLine("Waiting for NzbDrone to start. Response Status : {0}  [{1}] {2}", statusCall.ResponseStatus, statusCall.StatusDescription, statusCall.ErrorException);

                Thread.Sleep(500);
            }
        }

        public void KillAll()
        {
            if (_nzbDroneProcess != null)
            {
                _processProvider.Kill(_nzbDroneProcess.Id);
            }

            _processProvider.KillAll(ProcessProvider.NZB_DRONE_CONSOLE_PROCESS_NAME);
            _processProvider.KillAll(ProcessProvider.NZB_DRONE_PROCESS_NAME);
        }

        private void Start(string outputNzbdroneConsoleExe)
        {
            var args = "-nobrowser -data=\"" + AppData + "\"";
            _nzbDroneProcess = _processProvider.Start(outputNzbdroneConsoleExe, args, null, OnOutputDataReceived, OnOutputDataReceived);

        }

        private void OnOutputDataReceived(string data)
        {
            Console.WriteLine(data);

            if (data.Contains("Press enter to exit"))
            {
                _nzbDroneProcess.StandardInput.WriteLine(" ");
            }
        }

        private void SetApiKey()
        {
            var configFile = Path.Combine(AppData, "config.xml");
            var attempts = 0;

            while (ApiKey == null && attempts < 50)
            {
                try
                {
                    if (File.Exists(configFile))
                    {
                        var apiKeyElement = XDocument.Load(configFile)
                            .XPathSelectElement("Config/ApiKey");
                        if (apiKeyElement != null)
                        {
                            ApiKey = apiKeyElement.Value;
                        }
                    }
                }
                catch (XmlException ex)
                {
                    Console.WriteLine("Error getting API Key from XML file: " + ex.Message, ex);
                }

                attempts++;
                Thread.Sleep(1000);
            }
        }
    }
}

==> v0.6.0.815.zip: lidarr/src/NzbDrone.Test.Common/NzbDroneRunner.cs <==
using System;
using System.Diagnostics;
using System.IO;
using System.Threading;
using System.Xml;
using System.Xml.Linq;
using System.Xml.XPath;
using NLog;
using NUnit.Framework;
using NzbDrone.Common.EnvironmentInfo;
using NzbDrone.Common.Processes;
using RestSharp;

namespace NzbDrone.Test.Common
{
    public class NzbDroneRunner
    {
        private readonly IProcessProvider _processProvider;
        private readonly IRestClient _restClient;
        private Process _nzbDroneProcess;

        public string AppData { get; private set; }
        public string ApiKey { get; private set; }

        public NzbDroneRunner(Logger logger, int port = 8686)
        {
            _processProvider = new ProcessProvider(logger);
            _restClient = new RestClient("http://localhost:8686/api");
        }

        public void Start()
        {
            AppData = Path.Combine(TestContext.CurrentContext.TestDirectory, "_intg_" + DateTime.Now.Ticks);

            var lidarrConsoleExe = OsInfo.IsWindows ? "Lidarr.Console.exe" : "Lidarr.exe";

            if (BuildInfo.IsDebug)
            {
                Start(Path.Combine(TestContext.CurrentContext.TestDirectory, "..\\..\\..\\..\\..\\_output\\Lidarr.Console.exe"));
            }
            else
            {
                Start(Path.Combine("bin", lidarrConsoleExe));
            }

            while (true)
            {
                _nzbDroneProcess.Refresh();

                if (_nzbDroneProcess.HasExited)
                {
                    Assert.Fail("Process has exited");
                }

                SetApiKey();

                var request = new RestRequest("system/status");
                request.AddHeader("Authorization", ApiKey);
                request.AddHeader("X-Api-Key", ApiKey);

                var statusCall = _restClient.Get(request);

                if (statusCall.ResponseStatus == ResponseStatus.Completed)
                {
                    Console.WriteLine("Lidarr is started. Running Tests");
                    return;
                }

                Console.WriteLine("Waiting for Lidarr to start. Response Status : {0}  [{1}] {2}", statusCall.ResponseStatus, statusCall.StatusDescription, statusCall.ErrorException);

                Thread.Sleep(500);
            }
        }

        public void KillAll()
        {
            if (_nzbDroneProcess != null)
            {
                _processProvider.Kill(_nzbDroneProcess.Id);                
            }

            _processProvider.KillAll(ProcessProvider.LIDARR_CONSOLE_PROCESS_NAME);
            _processProvider.KillAll(ProcessProvider.LIDARR_PROCESS_NAME);
        }

        private void Start(string outputNzbdroneConsoleExe)
        {
            var args = "-nobrowser -data=\"" + AppData + "\"";
            _nzbDroneProcess = _processProvider.Start(outputNzbdroneConsoleExe, args, null, OnOutputDataReceived, OnOutputDataReceived);

        }

        private void OnOutputDataReceived(string data)
        {
            Console.WriteLine(data);

            if (data.Contains("Press enter to exit"))
            {
                _nzbDroneProcess.StandardInput.WriteLine(" ");
            }
        }

        private void SetApiKey()
        {
            var configFile = Path.Combine(AppData, "config.xml");
            var attempts = 0;

            while (ApiKey == null && attempts < 50)
            {
                try
                {
                    if (File.Exists(configFile))
                    {
                        var apiKeyElement = XDocument.Load(configFile)
                            .XPathSelectElement("Config/ApiKey");
                        if (apiKeyElement != null)
                        {
                            ApiKey = apiKeyElement.Value;
                        }
                    }
                }
                catch (XmlException ex)
                {
                    Console.WriteLine("Error getting API Key from XML file: " + ex.Message, ex);
                }

                attempts++;
                Thread.Sleep(1000);
            }
        }
    }
}
